
// REQUIRES the following Arduino libraries:
// - DHT Sensor Library: https://github.com/adafruit/DHT-sensor-library
// - Adafruit Unified Sensor Lib: https://github.com/adafruit/Adafruit_Sensor

#define DHTPIN 2     // Digital pin connected to the DHT sensor

#define LED 5  // The pin the LED is connected to

#define DHTTYPE DHT11   // DHT 11
//#define DHTTYPE DHT22   // DHT 22  (AM2302), AM2321
//#define DHTTYPE DHT21   // DHT 21 (AM2301)

#define USE_ARDUINO_INTERRUPTS true    // Set-up low-level interrupts for most acurate BPM math.

#include "DHT.h"
#include <PulseSensorPlayground.h>     // Includes the PulseSensorPlayground Library.   



// Connect pin 1 (on the left) of the sensor to +5V
// NOTE: If using a board with 3.3V logic like an Arduino Due connect pin 1
// to 3.3V instead of 5V!
// Connect pin 2 of the sensor to whatever your DHTPIN is
// Connect pin 4 (on the right) of the sensor to GROUND
// Connect a 10K resistor from pin 2 (data) to pin 1 (power) of the sensor

// Initialize DHT sensor.
// Note that older versions of this library took an optional third parameter to
// tweak the timings for faster processors.  This parameter is no longer needed
// as the current DHT reading algorithm adjusts itself to work on faster procs.

DHT dht(DHTPIN, DHTTYPE);

const int PulseWire = 0;       // PulseSensor PURPLE WIRE connected to ANALOG PIN 0
const int LED13 = 13;          // The on-board Arduino LED, close to PIN 13.
int Threshold = 513;           // Determine which Signal to "count as a beat" and which to ignore.
                               // Use the "Gettting Started Project" to fine-tune Threshold Value beyond default setting.
                               // Otherwise leave the default "550" value. 

int prev_bpm = 0;
bool firstime = 1;

PulseSensorPlayground pulseSensor;  // Creates an instance of the PulseSensorPlayground object called "pulseSensor"


void setup() {
  Serial.begin(9600);
  
  Serial.println(F("DHTxx test!"));
  dht.begin();

  pinMode(LED, OUTPUT); // Declare the LED as an output

  pulseSensor.analogInput(PulseWire);   
  pulseSensor.blinkOnPulse(LED13);       //auto-magically blink Arduino's LED with heartbeat.
  pulseSensor.setThreshold(Threshold);   

  // Double-check the "pulseSensor" object was created and "began" seeing a signal. 
   if (pulseSensor.begin()) {
    Serial.println("We created a pulseSensor Object !");  //This prints one time at Arduino power-up,  or on Arduino reset.  
  }
}

void loop() {
  // Wait a few seconds between measurements.
  //digitalWrite(LED, HIGH); // Turn the LED on
  
  delay(2000);

  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();
  // Read temperature as Fahrenheit (isFahrenheit = true)
  float f = dht.readTemperature(true);

  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t) || isnan(f)) {
    Serial.println(F("Failed to read from DHT sensor!"));
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  float hif = dht.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);

  float avg_temp, avg_hum, avg_bpm, hic_avg;
  int myBPM = pulseSensor.getBeatsPerMinute();  // Calls function on our pulseSensor object that returns BPM as an "int".
  int counter = 10;

  if (firstime){
    Serial.println("Etapa de configurare:");
    delay(900);
    while(counter--)
    {
        myBPM = pulseSensor.getBeatsPerMinute();
        Serial.print("BPM: ");                        // Print phrase "BPM: " 
        Serial.println(myBPM);
        delay(1000);
    }

    Serial.println("Intrebarea 1: Este adevarat ca esti student?");
    float h1 = dht.readHumidity();
    float t1 = dht.readTemperature();
    delay(3000);
    Serial.println("Intreabrarea 2: Este adevarat ca iti place PM?");
    float h2 = dht.readHumidity();
    float t2 = dht.readTemperature();
    delay(3000);
    Serial.println("\nCalculare valori...\n");
    avg_temp = (t1 + t2)/2;
    avg_hum = (h1 + h2)/2;
    hic_avg = dht.computeHeatIndex(avg_temp, avg_hum, false);

    firstime = 0;
  }

  Serial.print("Humidity: ");
  Serial.println(h);
  Serial.print("Temperature: ");
  Serial.print(t);
  Serial.print("\r");
  Serial.println("C ");
  Serial.print("Heat index: ");
  Serial.print(hic);
  Serial.println("C \n");
                                               // "myBPM" hold this BPM value now.
  if (myBPM - prev_bpm >= 10 && hic > hic_avg){
     digitalWrite(LED, HIGH); // Turn the LED on
     delay(50);
     digitalWrite(LED, LOW); // Turn the LED on
  }
  prev_bpm = myBPM;

   Serial.print("BPM: ");                        // Print phrase "BPM: " 
   Serial.println(myBPM);

if (pulseSensor.sawStartOfBeat() && myBPM < 140 ) {            // Constantly test to see if "a beat happened". 
 Serial.println("♥  A HeartBeat Happened ! "); // If test is "true", print a message "a heartbeat happened".
 Serial.print("BPM: ");                        // Print phrase "BPM: " 
 Serial.println(myBPM);                        // Print the value inside of myBPM. 
}

  delay(20);                    // considered best practice in a simple sketch.
}
